from enum import Enum


class PostRunCheckStatus(str, Enum):
    CANCELLED = "cancelled"
    FAILED = "failed"
    INFRA_ERROR = "infra_error"
    PENDING = "pending"
    RUNNING = "running"
    SUCCESS = "success"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowDuplicateRequest")


@_attrs_define
class WorkflowDuplicateRequest:
    """Schema for duplicating workflows.

    Attributes:
        include_trajectories (bool | Unset): Whether to duplicate generated trajectories onto the copied workflow. Non-
            generated trajectories are never duplicated. Default: False.
    """

    include_trajectories: bool | Unset = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        include_trajectories = self.include_trajectories

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if include_trajectories is not UNSET:
            field_dict["include_trajectories"] = include_trajectories

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        include_trajectories = d.pop("include_trajectories", UNSET)

        workflow_duplicate_request = cls(
            include_trajectories=include_trajectories,
        )

        workflow_duplicate_request.additional_properties = d
        return workflow_duplicate_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowInternalMetadataUpdate")


@_attrs_define
class WorkflowInternalMetadataUpdate:
    """Patchable workflow metadata fields for internal workflow maintenance.

    Attributes:
        has_unaddressed_error (bool | None | Unset): Set to false when workflow errors have been addressed.
        last_errored_run_start (datetime.datetime | None | Unset): Start timestamp of the most recent
            errored/task_failed run.
        last_errored_run_uuid (None | Unset | UUID): Run UUID of the most recent errored/task_failed run.
    """

    has_unaddressed_error: bool | None | Unset = UNSET
    last_errored_run_start: datetime.datetime | None | Unset = UNSET
    last_errored_run_uuid: None | Unset | UUID = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        has_unaddressed_error: bool | None | Unset
        if isinstance(self.has_unaddressed_error, Unset):
            has_unaddressed_error = UNSET
        else:
            has_unaddressed_error = self.has_unaddressed_error

        last_errored_run_start: None | str | Unset
        if isinstance(self.last_errored_run_start, Unset):
            last_errored_run_start = UNSET
        elif isinstance(self.last_errored_run_start, datetime.datetime):
            last_errored_run_start = self.last_errored_run_start.isoformat()
        else:
            last_errored_run_start = self.last_errored_run_start

        last_errored_run_uuid: None | str | Unset
        if isinstance(self.last_errored_run_uuid, Unset):
            last_errored_run_uuid = UNSET
        elif isinstance(self.last_errored_run_uuid, UUID):
            last_errored_run_uuid = str(self.last_errored_run_uuid)
        else:
            last_errored_run_uuid = self.last_errored_run_uuid

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if has_unaddressed_error is not UNSET:
            field_dict["has_unaddressed_error"] = has_unaddressed_error
        if last_errored_run_start is not UNSET:
            field_dict["last_errored_run_start"] = last_errored_run_start
        if last_errored_run_uuid is not UNSET:
            field_dict["last_errored_run_uuid"] = last_errored_run_uuid

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_has_unaddressed_error(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        has_unaddressed_error = _parse_has_unaddressed_error(d.pop("has_unaddressed_error", UNSET))

        def _parse_last_errored_run_start(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_errored_run_start_type_0 = isoparse(data)

                return last_errored_run_start_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        last_errored_run_start = _parse_last_errored_run_start(d.pop("last_errored_run_start", UNSET))

        def _parse_last_errored_run_uuid(data: object) -> None | Unset | UUID:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_errored_run_uuid_type_0 = UUID(data)

                return last_errored_run_uuid_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UUID, data)

        last_errored_run_uuid = _parse_last_errored_run_uuid(d.pop("last_errored_run_uuid", UNSET))

        workflow_internal_metadata_update = cls(
            has_unaddressed_error=has_unaddressed_error,
            last_errored_run_start=last_errored_run_start,
            last_errored_run_uuid=last_errored_run_uuid,
        )

        workflow_internal_metadata_update.additional_properties = d
        return workflow_internal_metadata_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any
from urllib.parse import quote
from uuid import UUID

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.run_attachment_download_url_response import RunAttachmentDownloadUrlResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    attachment_id: UUID,
    *,
    expires_in: int | Unset = 300,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    params["expires_in"] = expires_in

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/run-attachments/{attachment_id}/download-url".format(
            attachment_id=quote(str(attachment_id), safe=""),
        ),
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> HTTPValidationError | RunAttachmentDownloadUrlResponse | None:
    if response.status_code == 200:
        response_200 = RunAttachmentDownloadUrlResponse.from_dict(response.json())

        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[HTTPValidationError | RunAttachmentDownloadUrlResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    attachment_id: UUID,
    *,
    client: AuthenticatedClient,
    expires_in: int | Unset = 300,
) -> Response[HTTPValidationError | RunAttachmentDownloadUrlResponse]:
    """Get Run Attachment Download Url

     Get a signed download URL for a run attachment file.

    Returns a signed URL that triggers automatic download when accessed.

    Args:
        attachment_id: The ID of the attachment to download
        expires_in: URL expiration time in seconds (10-3600). Default: 300 (5 minutes)

    Args:
        attachment_id (UUID):
        expires_in (int | Unset): URL expiration time in seconds (10-3600) Default: 300.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | RunAttachmentDownloadUrlResponse]
    """

    kwargs = _get_kwargs(
        attachment_id=attachment_id,
        expires_in=expires_in,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    attachment_id: UUID,
    *,
    client: AuthenticatedClient,
    expires_in: int | Unset = 300,
) -> HTTPValidationError | RunAttachmentDownloadUrlResponse | None:
    """Get Run Attachment Download Url

     Get a signed download URL for a run attachment file.

    Returns a signed URL that triggers automatic download when accessed.

    Args:
        attachment_id: The ID of the attachment to download
        expires_in: URL expiration time in seconds (10-3600). Default: 300 (5 minutes)

    Args:
        attachment_id (UUID):
        expires_in (int | Unset): URL expiration time in seconds (10-3600) Default: 300.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | RunAttachmentDownloadUrlResponse
    """

    return sync_detailed(
        attachment_id=attachment_id,
        client=client,
        expires_in=expires_in,
    ).parsed


async def asyncio_detailed(
    attachment_id: UUID,
    *,
    client: AuthenticatedClient,
    expires_in: int | Unset = 300,
) -> Response[HTTPValidationError | RunAttachmentDownloadUrlResponse]:
    """Get Run Attachment Download Url

     Get a signed download URL for a run attachment file.

    Returns a signed URL that triggers automatic download when accessed.

    Args:
        attachment_id: The ID of the attachment to download
        expires_in: URL expiration time in seconds (10-3600). Default: 300 (5 minutes)

    Args:
        attachment_id (UUID):
        expires_in (int | Unset): URL expiration time in seconds (10-3600) Default: 300.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | RunAttachmentDownloadUrlResponse]
    """

    kwargs = _get_kwargs(
        attachment_id=attachment_id,
        expires_in=expires_in,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    attachment_id: UUID,
    *,
    client: AuthenticatedClient,
    expires_in: int | Unset = 300,
) -> HTTPValidationError | RunAttachmentDownloadUrlResponse | None:
    """Get Run Attachment Download Url

     Get a signed download URL for a run attachment file.

    Returns a signed URL that triggers automatic download when accessed.

    Args:
        attachment_id: The ID of the attachment to download
        expires_in: URL expiration time in seconds (10-3600). Default: 300 (5 minutes)

    Args:
        attachment_id (UUID):
        expires_in (int | Unset): URL expiration time in seconds (10-3600) Default: 300.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | RunAttachmentDownloadUrlResponse
    """

    return (
        await asyncio_detailed(
            attachment_id=attachment_id,
            client=client,
            expires_in=expires_in,
        )
    ).parsed



from enum import Enum


class MachinesHealthCheckResponseScope(str, Enum):
    ALL_ORGANIZATIONS = "all_organizations"
    ORGANIZATION = "organization"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any
from urllib.parse import quote
from uuid import UUID

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.pool_response_with_includes import PoolResponseWithIncludes
from ...types import UNSET, Response, Unset


def _get_kwargs(
    pool_id: UUID,
    *,
    include_machines: bool | Unset = False,
    include: None | str | Unset = UNSET,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    params["include_machines"] = include_machines

    json_include: None | str | Unset
    if isinstance(include, Unset):
        json_include = UNSET
    else:
        json_include = include
    params["include"] = json_include

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/pools/{pool_id}".format(
            pool_id=quote(str(pool_id), safe=""),
        ),
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> HTTPValidationError | PoolResponseWithIncludes | None:
    if response.status_code == 200:
        response_200 = PoolResponseWithIncludes.from_dict(response.json())

        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[HTTPValidationError | PoolResponseWithIncludes]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    pool_id: UUID,
    *,
    client: AuthenticatedClient,
    include_machines: bool | Unset = False,
    include: None | str | Unset = UNSET,
) -> Response[HTTPValidationError | PoolResponseWithIncludes]:
    """Get Pool

     Get a specific pool by ID.

    Use the `include` parameter to fetch related resources in the response.
    Related resources are returned in the `included` array following the JSON:API pattern.

    Note: The `include_machines` parameter is deprecated. Use `include=machines` instead.

    Args:
        pool_id (UUID):
        include_machines (bool | Unset): [Deprecated] Use include=machines instead. Include full
            machine details Default: False.
        include (None | str | Unset): Comma-separated list of related resources to include.
            Allowed values: machines. Example: include=machines

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | PoolResponseWithIncludes]
    """

    kwargs = _get_kwargs(
        pool_id=pool_id,
        include_machines=include_machines,
        include=include,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    pool_id: UUID,
    *,
    client: AuthenticatedClient,
    include_machines: bool | Unset = False,
    include: None | str | Unset = UNSET,
) -> HTTPValidationError | PoolResponseWithIncludes | None:
    """Get Pool

     Get a specific pool by ID.

    Use the `include` parameter to fetch related resources in the response.
    Related resources are returned in the `included` array following the JSON:API pattern.

    Note: The `include_machines` parameter is deprecated. Use `include=machines` instead.

    Args:
        pool_id (UUID):
        include_machines (bool | Unset): [Deprecated] Use include=machines instead. Include full
            machine details Default: False.
        include (None | str | Unset): Comma-separated list of related resources to include.
            Allowed values: machines. Example: include=machines

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | PoolResponseWithIncludes
    """

    return sync_detailed(
        pool_id=pool_id,
        client=client,
        include_machines=include_machines,
        include=include,
    ).parsed


async def asyncio_detailed(
    pool_id: UUID,
    *,
    client: AuthenticatedClient,
    include_machines: bool | Unset = False,
    include: None | str | Unset = UNSET,
) -> Response[HTTPValidationError | PoolResponseWithIncludes]:
    """Get Pool

     Get a specific pool by ID.

    Use the `include` parameter to fetch related resources in the response.
    Related resources are returned in the `included` array following the JSON:API pattern.

    Note: The `include_machines` parameter is deprecated. Use `include=machines` instead.

    Args:
        pool_id (UUID):
        include_machines (bool | Unset): [Deprecated] Use include=machines instead. Include full
            machine details Default: False.
        include (None | str | Unset): Comma-separated list of related resources to include.
            Allowed values: machines. Example: include=machines

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | PoolResponseWithIncludes]
    """

    kwargs = _get_kwargs(
        pool_id=pool_id,
        include_machines=include_machines,
        include=include,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    pool_id: UUID,
    *,
    client: AuthenticatedClient,
    include_machines: bool | Unset = False,
    include: None | str | Unset = UNSET,
) -> HTTPValidationError | PoolResponseWithIncludes | None:
    """Get Pool

     Get a specific pool by ID.

    Use the `include` parameter to fetch related resources in the response.
    Related resources are returned in the `included` array following the JSON:API pattern.

    Note: The `include_machines` parameter is deprecated. Use `include=machines` instead.

    Args:
        pool_id (UUID):
        include_machines (bool | Unset): [Deprecated] Use include=machines instead. Include full
            machine details Default: False.
        include (None | str | Unset): Comma-separated list of related resources to include.
            Allowed values: machines. Example: include=machines

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | PoolResponseWithIncludes
    """

    return (
        await asyncio_detailed(
            pool_id=pool_id,
            client=client,
            include_machines=include_machines,
            include=include,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowMergeRequest")


@_attrs_define
class WorkflowMergeRequest:
    """Schema for merging workflows

    Attributes:
        target_workflow_id (UUID): Workflow ID to merge into
        copy_trajectories (bool | Unset): Whether to copy trajectories to the target workflow Default: False.
    """

    target_workflow_id: UUID
    copy_trajectories: bool | Unset = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        target_workflow_id = str(self.target_workflow_id)

        copy_trajectories = self.copy_trajectories

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "target_workflow_id": target_workflow_id,
            }
        )
        if copy_trajectories is not UNSET:
            field_dict["copy_trajectories"] = copy_trajectories

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        target_workflow_id = UUID(d.pop("target_workflow_id"))

        copy_trajectories = d.pop("copy_trajectories", UNSET)

        workflow_merge_request = cls(
            target_workflow_id=target_workflow_id,
            copy_trajectories=copy_trajectories,
        )

        workflow_merge_request.additional_properties = d
        return workflow_merge_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class PostRunCheckType(str, Enum):
    OUTPUT_DATA_AGENTIC_CHECK = "output_data_agentic_check"
    OUTPUT_DATA_PASSES_SCHEMA_VALIDATION = "output_data_passes_schema_validation"
    RUN_ATTACHMENT_EXISTS = "run_attachment_exists"
    RUN_ATTACHMENT_IMAGE_CHECK = "run_attachment_image_check"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.machines_health_check_response_scope import MachinesHealthCheckResponseScope
from ..models.machines_health_check_response_status import MachinesHealthCheckResponseStatus

if TYPE_CHECKING:
    from ..models.machine_health_check_error import MachineHealthCheckError


T = TypeVar("T", bound="MachinesHealthCheckResponse")


@_attrs_define
class MachinesHealthCheckResponse:
    """
    Attributes:
        status (MachinesHealthCheckResponseStatus):
        scope (MachinesHealthCheckResponseScope):
        checked_machines (int):
        healthy_machines (int):
        errored_machines (list[MachineHealthCheckError]):
        message (str):
    """

    status: MachinesHealthCheckResponseStatus
    scope: MachinesHealthCheckResponseScope
    checked_machines: int
    healthy_machines: int
    errored_machines: list[MachineHealthCheckError]
    message: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        status = self.status.value

        scope = self.scope.value

        checked_machines = self.checked_machines

        healthy_machines = self.healthy_machines

        errored_machines = []
        for errored_machines_item_data in self.errored_machines:
            errored_machines_item = errored_machines_item_data.to_dict()
            errored_machines.append(errored_machines_item)

        message = self.message

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "status": status,
                "scope": scope,
                "checked_machines": checked_machines,
                "healthy_machines": healthy_machines,
                "errored_machines": errored_machines,
                "message": message,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.machine_health_check_error import MachineHealthCheckError

        d = dict(src_dict)
        status = MachinesHealthCheckResponseStatus(d.pop("status"))

        scope = MachinesHealthCheckResponseScope(d.pop("scope"))

        checked_machines = d.pop("checked_machines")

        healthy_machines = d.pop("healthy_machines")

        errored_machines = []
        _errored_machines = d.pop("errored_machines")
        for errored_machines_item_data in _errored_machines:
            errored_machines_item = MachineHealthCheckError.from_dict(errored_machines_item_data)

            errored_machines.append(errored_machines_item)

        message = d.pop("message")

        machines_health_check_response = cls(
            status=status,
            scope=scope,
            checked_machines=checked_machines,
            healthy_machines=healthy_machines,
            errored_machines=errored_machines,
            message=message,
        )

        machines_health_check_response.additional_properties = d
        return machines_health_check_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="CyberdriverShutdownRequest")


@_attrs_define
class CyberdriverShutdownRequest:
    """
    Attributes:
        source (str | Unset): Caller identifier for shutdown auditing Default: 'cloud-api'.
        reason (None | str | Unset): Reason for shutting down the CyberDriver process Default: 'user_disabled_machine'.
    """

    source: str | Unset = "cloud-api"
    reason: None | str | Unset = "user_disabled_machine"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        source = self.source

        reason: None | str | Unset
        if isinstance(self.reason, Unset):
            reason = UNSET
        else:
            reason = self.reason

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if source is not UNSET:
            field_dict["source"] = source
        if reason is not UNSET:
            field_dict["reason"] = reason

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        source = d.pop("source", UNSET)

        def _parse_reason(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        reason = _parse_reason(d.pop("reason", UNSET))

        cyberdriver_shutdown_request = cls(
            source=source,
            reason=reason,
        )

        cyberdriver_shutdown_request.additional_properties = d
        return cyberdriver_shutdown_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="MachineHealthCheckError")


@_attrs_define
class MachineHealthCheckError:
    """
    Attributes:
        machine_id (str):
        error (str):
        organization_id (None | str | Unset):
        machine_name (None | str | Unset):
        status_code (int | None | Unset):
    """

    machine_id: str
    error: str
    organization_id: None | str | Unset = UNSET
    machine_name: None | str | Unset = UNSET
    status_code: int | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        machine_id = self.machine_id

        error = self.error

        organization_id: None | str | Unset
        if isinstance(self.organization_id, Unset):
            organization_id = UNSET
        else:
            organization_id = self.organization_id

        machine_name: None | str | Unset
        if isinstance(self.machine_name, Unset):
            machine_name = UNSET
        else:
            machine_name = self.machine_name

        status_code: int | None | Unset
        if isinstance(self.status_code, Unset):
            status_code = UNSET
        else:
            status_code = self.status_code

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "machine_id": machine_id,
                "error": error,
            }
        )
        if organization_id is not UNSET:
            field_dict["organization_id"] = organization_id
        if machine_name is not UNSET:
            field_dict["machine_name"] = machine_name
        if status_code is not UNSET:
            field_dict["status_code"] = status_code

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        machine_id = d.pop("machine_id")

        error = d.pop("error")

        def _parse_organization_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        organization_id = _parse_organization_id(d.pop("organization_id", UNSET))

        def _parse_machine_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        machine_name = _parse_machine_name(d.pop("machine_name", UNSET))

        def _parse_status_code(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        status_code = _parse_status_code(d.pop("status_code", UNSET))

        machine_health_check_error = cls(
            machine_id=machine_id,
            error=error,
            organization_id=organization_id,
            machine_name=machine_name,
            status_code=status_code,
        )

        machine_health_check_error.additional_properties = d
        return machine_health_check_error

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.post_run_check_file_target_mode import PostRunCheckFileTargetMode
from ..models.post_run_check_type import PostRunCheckType
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowPostRunCheckUpsert")


@_attrs_define
class WorkflowPostRunCheckUpsert:
    """Create/update payload for workflow-managed post-run checks.

    Attributes:
        name (str):
        type_ (PostRunCheckType):
        description (None | str | Unset):
        file_target_mode (None | PostRunCheckFileTargetMode | Unset):
        file_names (list[str] | None | Unset):
        file_name_regex (None | str | Unset):
        expected_match_count (int | None | Unset):
        expected_match_count_ref (None | str | Unset):
        loop_input (None | str | Unset):
        loop_item_filename_template (None | str | Unset):
        check_prompt (None | str | Unset):
        model (None | Unset | UUID): Optional ModelConfiguration id override for agentic post-run checks.
        id (None | Unset | UUID):
    """

    name: str
    type_: PostRunCheckType
    description: None | str | Unset = UNSET
    file_target_mode: None | PostRunCheckFileTargetMode | Unset = UNSET
    file_names: list[str] | None | Unset = UNSET
    file_name_regex: None | str | Unset = UNSET
    expected_match_count: int | None | Unset = UNSET
    expected_match_count_ref: None | str | Unset = UNSET
    loop_input: None | str | Unset = UNSET
    loop_item_filename_template: None | str | Unset = UNSET
    check_prompt: None | str | Unset = UNSET
    model: None | Unset | UUID = UNSET
    id: None | Unset | UUID = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        type_ = self.type_.value

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        file_target_mode: None | str | Unset
        if isinstance(self.file_target_mode, Unset):
            file_target_mode = UNSET
        elif isinstance(self.file_target_mode, PostRunCheckFileTargetMode):
            file_target_mode = self.file_target_mode.value
        else:
            file_target_mode = self.file_target_mode

        file_names: list[str] | None | Unset
        if isinstance(self.file_names, Unset):
            file_names = UNSET
        elif isinstance(self.file_names, list):
            file_names = self.file_names

        else:
            file_names = self.file_names

        file_name_regex: None | str | Unset
        if isinstance(self.file_name_regex, Unset):
            file_name_regex = UNSET
        else:
            file_name_regex = self.file_name_regex

        expected_match_count: int | None | Unset
        if isinstance(self.expected_match_count, Unset):
            expected_match_count = UNSET
        else:
            expected_match_count = self.expected_match_count

        expected_match_count_ref: None | str | Unset
        if isinstance(self.expected_match_count_ref, Unset):
            expected_match_count_ref = UNSET
        else:
            expected_match_count_ref = self.expected_match_count_ref

        loop_input: None | str | Unset
        if isinstance(self.loop_input, Unset):
            loop_input = UNSET
        else:
            loop_input = self.loop_input

        loop_item_filename_template: None | str | Unset
        if isinstance(self.loop_item_filename_template, Unset):
            loop_item_filename_template = UNSET
        else:
            loop_item_filename_template = self.loop_item_filename_template

        check_prompt: None | str | Unset
        if isinstance(self.check_prompt, Unset):
            check_prompt = UNSET
        else:
            check_prompt = self.check_prompt

        model: None | str | Unset
        if isinstance(self.model, Unset):
            model = UNSET
        elif isinstance(self.model, UUID):
            model = str(self.model)
        else:
            model = self.model

        id: None | str | Unset
        if isinstance(self.id, Unset):
            id = UNSET
        elif isinstance(self.id, UUID):
            id = str(self.id)
        else:
            id = self.id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "type": type_,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if file_target_mode is not UNSET:
            field_dict["file_target_mode"] = file_target_mode
        if file_names is not UNSET:
            field_dict["file_names"] = file_names
        if file_name_regex is not UNSET:
            field_dict["file_name_regex"] = file_name_regex
        if expected_match_count is not UNSET:
            field_dict["expected_match_count"] = expected_match_count
        if expected_match_count_ref is not UNSET:
            field_dict["expected_match_count_ref"] = expected_match_count_ref
        if loop_input is not UNSET:
            field_dict["loop_input"] = loop_input
        if loop_item_filename_template is not UNSET:
            field_dict["loop_item_filename_template"] = loop_item_filename_template
        if check_prompt is not UNSET:
            field_dict["check_prompt"] = check_prompt
        if model is not UNSET:
            field_dict["model"] = model
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        type_ = PostRunCheckType(d.pop("type"))

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_file_target_mode(data: object) -> None | PostRunCheckFileTargetMode | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                file_target_mode_type_0 = PostRunCheckFileTargetMode(data)

                return file_target_mode_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | PostRunCheckFileTargetMode | Unset, data)

        file_target_mode = _parse_file_target_mode(d.pop("file_target_mode", UNSET))

        def _parse_file_names(data: object) -> list[str] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                file_names_type_0 = cast(list[str], data)

                return file_names_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None | Unset, data)

        file_names = _parse_file_names(d.pop("file_names", UNSET))

        def _parse_file_name_regex(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        file_name_regex = _parse_file_name_regex(d.pop("file_name_regex", UNSET))

        def _parse_expected_match_count(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        expected_match_count = _parse_expected_match_count(d.pop("expected_match_count", UNSET))

        def _parse_expected_match_count_ref(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        expected_match_count_ref = _parse_expected_match_count_ref(d.pop("expected_match_count_ref", UNSET))

        def _parse_loop_input(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        loop_input = _parse_loop_input(d.pop("loop_input", UNSET))

        def _parse_loop_item_filename_template(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        loop_item_filename_template = _parse_loop_item_filename_template(d.pop("loop_item_filename_template", UNSET))

        def _parse_check_prompt(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        check_prompt = _parse_check_prompt(d.pop("check_prompt", UNSET))

        def _parse_model(data: object) -> None | Unset | UUID:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                model_type_0 = UUID(data)

                return model_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UUID, data)

        model = _parse_model(d.pop("model", UNSET))

        def _parse_id(data: object) -> None | Unset | UUID:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                id_type_0 = UUID(data)

                return id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UUID, data)

        id = _parse_id(d.pop("id", UNSET))

        workflow_post_run_check_upsert = cls(
            name=name,
            type_=type_,
            description=description,
            file_target_mode=file_target_mode,
            file_names=file_names,
            file_name_regex=file_name_regex,
            expected_match_count=expected_match_count,
            expected_match_count_ref=expected_match_count_ref,
            loop_input=loop_input,
            loop_item_filename_template=loop_item_filename_template,
            check_prompt=check_prompt,
            model=model,
            id=id,
        )

        workflow_post_run_check_upsert.additional_properties = d
        return workflow_post_run_check_upsert

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.run_status import RunStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.included_resource import IncludedResource
    from ..models.run_post_run_check_snapshot import RunPostRunCheckSnapshot
    from ..models.run_response_with_includes_input_values_type_0 import RunResponseWithIncludesInputValuesType0
    from ..models.run_response_with_includes_output_data_type_0 import RunResponseWithIncludesOutputDataType0
    from ..models.run_response_with_includes_run_message_history_type_0_item import (
        RunResponseWithIncludesRunMessageHistoryType0Item,
    )
    from ..models.run_response_with_includes_sensitive_input_aliases_type_0 import (
        RunResponseWithIncludesSensitiveInputAliasesType0,
    )
    from ..models.run_response_with_includes_usage_metadata_type_0 import RunResponseWithIncludesUsageMetadataType0


T = TypeVar("T", bound="RunResponseWithIncludes")


@_attrs_define
class RunResponseWithIncludes:
    """Run response with optional included related resources.

    When the `include` query parameter is used, related resources are returned
    in the `included` array following the JSON:API pattern.

    Example request: GET /v1/runs/123?include=workflow,machine

        Attributes:
            id (UUID):
            workflow_id (UUID):
            status (RunStatus):
            created_at (datetime.datetime):
            machine_id (None | Unset | UUID):
            user_id (None | Unset | UUID):
            organization_id (None | str | Unset):
            error (list[str] | None | Unset):
            output_data (None | RunResponseWithIncludesOutputDataType0 | Unset):
            input_attachment_ids (list[str] | None | Unset):
            output_attachment_ids (list[str] | None | Unset):
            run_message_history (list[RunResponseWithIncludesRunMessageHistoryType0Item] | None | Unset):
            input_values (None | RunResponseWithIncludesInputValuesType0 | Unset):
            pool_ids (list[UUID] | None | Unset):
            sensitive_input_aliases (None | RunResponseWithIncludesSensitiveInputAliasesType0 | Unset):
            usage_metadata (None | RunResponseWithIncludesUsageMetadataType0 | Unset): Arbitrary usage/billing metadata
                captured during a run
            post_run_checks (list[RunPostRunCheckSnapshot] | None | Unset): Version-tolerant snapshot/results for the run's
                post-run checks.
            session_id (None | Unset | UUID):
            session_alias (None | str | Unset):
            release_session_after (bool | None | Unset):
            started_at (datetime.datetime | None | Unset):
            ended_at (datetime.datetime | None | Unset):
            included (list[IncludedResource] | None | Unset): Related resources requested via the `include` query parameter
    """

    id: UUID
    workflow_id: UUID
    status: RunStatus
    created_at: datetime.datetime
    machine_id: None | Unset | UUID = UNSET
    user_id: None | Unset | UUID = UNSET
    organization_id: None | str | Unset = UNSET
    error: list[str] | None | Unset = UNSET
    output_data: None | RunResponseWithIncludesOutputDataType0 | Unset = UNSET
    input_attachment_ids: list[str] | None | Unset = UNSET
    output_attachment_ids: list[str] | None | Unset = UNSET
    run_message_history: list[RunResponseWithIncludesRunMessageHistoryType0Item] | None | Unset = UNSET
    input_values: None | RunResponseWithIncludesInputValuesType0 | Unset = UNSET
    pool_ids: list[UUID] | None | Unset = UNSET
    sensitive_input_aliases: None | RunResponseWithIncludesSensitiveInputAliasesType0 | Unset = UNSET
    usage_metadata: None | RunResponseWithIncludesUsageMetadataType0 | Unset = UNSET
    post_run_checks: list[RunPostRunCheckSnapshot] | None | Unset = UNSET
    session_id: None | Unset | UUID = UNSET
    session_alias: None | str | Unset = UNSET
    release_session_after: bool | None | Unset = UNSET
    started_at: datetime.datetime | None | Unset = UNSET
    ended_at: datetime.datetime | None | Unset = UNSET
    included: list[IncludedResource] | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.run_response_with_includes_input_values_type_0 import RunResponseWithIncludesInputValuesType0
        from ..models.run_response_with_includes_output_data_type_0 import RunResponseWithIncludesOutputDataType0
        from ..models.run_response_with_includes_sensitive_input_aliases_type_0 import (
            RunResponseWithIncludesSensitiveInputAliasesType0,
        )
        from ..models.run_response_with_includes_usage_metadata_type_0 import RunResponseWithIncludesUsageMetadataType0

        id = str(self.id)

        workflow_id = str(self.workflow_id)

        status = self.status.value

        created_at = self.created_at.isoformat()

        machine_id: None | str | Unset
        if isinstance(self.machine_id, Unset):
            machine_id = UNSET
        elif isinstance(self.machine_id, UUID):
            machine_id = str(self.machine_id)
        else:
            machine_id = self.machine_id

        user_id: None | str | Unset
        if isinstance(self.user_id, Unset):
            user_id = UNSET
        elif isinstance(self.user_id, UUID):
            user_id = str(self.user_id)
        else:
            user_id = self.user_id

        organization_id: None | str | Unset
        if isinstance(self.organization_id, Unset):
            organization_id = UNSET
        else:
            organization_id = self.organization_id

        error: list[str] | None | Unset
        if isinstance(self.error, Unset):
            error = UNSET
        elif isinstance(self.error, list):
            error = self.error

        else:
            error = self.error

        output_data: dict[str, Any] | None | Unset
        if isinstance(self.output_data, Unset):
            output_data = UNSET
        elif isinstance(self.output_data, RunResponseWithIncludesOutputDataType0):
            output_data = self.output_data.to_dict()
        else:
            output_data = self.output_data

        input_attachment_ids: list[str] | None | Unset
        if isinstance(self.input_attachment_ids, Unset):
            input_attachment_ids = UNSET
        elif isinstance(self.input_attachment_ids, list):
            input_attachment_ids = self.input_attachment_ids

        else:
            input_attachment_ids = self.input_attachment_ids

        output_attachment_ids: list[str] | None | Unset
        if isinstance(self.output_attachment_ids, Unset):
            output_attachment_ids = UNSET
        elif isinstance(self.output_attachment_ids, list):
            output_attachment_ids = self.output_attachment_ids

        else:
            output_attachment_ids = self.output_attachment_ids

        run_message_history: list[dict[str, Any]] | None | Unset
        if isinstance(self.run_message_history, Unset):
            run_message_history = UNSET
        elif isinstance(self.run_message_history, list):
            run_message_history = []
            for run_message_history_type_0_item_data in self.run_message_history:
                run_message_history_type_0_item = run_message_history_type_0_item_data.to_dict()
                run_message_history.append(run_message_history_type_0_item)

        else:
            run_message_history = self.run_message_history

        input_values: dict[str, Any] | None | Unset
        if isinstance(self.input_values, Unset):
            input_values = UNSET
        elif isinstance(self.input_values, RunResponseWithIncludesInputValuesType0):
            input_values = self.input_values.to_dict()
        else:
            input_values = self.input_values

        pool_ids: list[str] | None | Unset
        if isinstance(self.pool_ids, Unset):
            pool_ids = UNSET
        elif isinstance(self.pool_ids, list):
            pool_ids = []
            for pool_ids_type_0_item_data in self.pool_ids:
                pool_ids_type_0_item = str(pool_ids_type_0_item_data)
                pool_ids.append(pool_ids_type_0_item)

        else:
            pool_ids = self.pool_ids

        sensitive_input_aliases: dict[str, Any] | None | Unset
        if isinstance(self.sensitive_input_aliases, Unset):
            sensitive_input_aliases = UNSET
        elif isinstance(self.sensitive_input_aliases, RunResponseWithIncludesSensitiveInputAliasesType0):
            sensitive_input_aliases = self.sensitive_input_aliases.to_dict()
        else:
            sensitive_input_aliases = self.sensitive_input_aliases

        usage_metadata: dict[str, Any] | None | Unset
        if isinstance(self.usage_metadata, Unset):
            usage_metadata = UNSET
        elif isinstance(self.usage_metadata, RunResponseWithIncludesUsageMetadataType0):
            usage_metadata = self.usage_metadata.to_dict()
        else:
            usage_metadata = self.usage_metadata

        post_run_checks: list[dict[str, Any]] | None | Unset
        if isinstance(self.post_run_checks, Unset):
            post_run_checks = UNSET
        elif isinstance(self.post_run_checks, list):
            post_run_checks = []
            for post_run_checks_type_0_item_data in self.post_run_checks:
                post_run_checks_type_0_item = post_run_checks_type_0_item_data.to_dict()
                post_run_checks.append(post_run_checks_type_0_item)

        else:
            post_run_checks = self.post_run_checks

        session_id: None | str | Unset
        if isinstance(self.session_id, Unset):
            session_id = UNSET
        elif isinstance(self.session_id, UUID):
            session_id = str(self.session_id)
        else:
            session_id = self.session_id

        session_alias: None | str | Unset
        if isinstance(self.session_alias, Unset):
            session_alias = UNSET
        else:
            session_alias = self.session_alias

        release_session_after: bool | None | Unset
        if isinstance(self.release_session_after, Unset):
            release_session_after = UNSET
        else:
            release_session_after = self.release_session_after

        started_at: None | str | Unset
        if isinstance(self.started_at, Unset):
            started_at = UNSET
        elif isinstance(self.started_at, datetime.datetime):
            started_at = self.started_at.isoformat()
        else:
            started_at = self.started_at

        ended_at: None | str | Unset
        if isinstance(self.ended_at, Unset):
            ended_at = UNSET
        elif isinstance(self.ended_at, datetime.datetime):
            ended_at = self.ended_at.isoformat()
        else:
            ended_at = self.ended_at

        included: list[dict[str, Any]] | None | Unset
        if isinstance(self.included, Unset):
            included = UNSET
        elif isinstance(self.included, list):
            included = []
            for included_type_0_item_data in self.included:
                included_type_0_item = included_type_0_item_data.to_dict()
                included.append(included_type_0_item)

        else:
            included = self.included

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "workflow_id": workflow_id,
                "status": status,
                "created_at": created_at,
            }
        )
        if machine_id is not UNSET:
            field_dict["machine_id"] = machine_id
        if user_id is not UNSET:
            field_dict["user_id"] = user_id
        if organization_id is not UNSET:
            field_dict["organization_id"] = organization_id
        if error is not UNSET:
            field_dict["error"] = error
        if output_data is not UNSET:
            field_dict["output_data"] = output_data
        if input_attachment_ids is not UNSET:
            field_dict["input_attachment_ids"] = input_attachment_ids
        if output_attachment_ids is not UNSET:
            field_dict["output_attachment_ids"] = output_attachment_ids
        if run_message_history is not UNSET:
            field_dict["run_message_history"] = run_message_history
        if input_values is not UNSET:
            field_dict["input_values"] = input_values
        if pool_ids is not UNSET:
            field_dict["pool_ids"] = pool_ids
        if sensitive_input_aliases is not UNSET:
            field_dict["sensitive_input_aliases"] = sensitive_input_aliases
        if usage_metadata is not UNSET:
            field_dict["usage_metadata"] = usage_metadata
        if post_run_checks is not UNSET:
            field_dict["post_run_checks"] = post_run_checks
        if session_id is not UNSET:
            field_dict["session_id"] = session_id
        if session_alias is not UNSET:
            field_dict["session_alias"] = session_alias
        if release_session_after is not UNSET:
            field_dict["release_session_after"] = release_session_after
        if started_at is not UNSET:
            field_dict["started_at"] = started_at
        if ended_at is not UNSET:
            field_dict["ended_at"] = ended_at
        if included is not UNSET:
            field_dict["included"] = included

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.included_resource import IncludedResource
        from ..models.run_post_run_check_snapshot import RunPostRunCheckSnapshot
        from ..models.run_response_with_includes_input_values_type_0 import RunResponseWithIncludesInputValuesType0
        from ..models.run_response_with_includes_output_data_type_0 import RunResponseWithIncludesOutputDataType0
        from ..models.run_response_with_includes_run_message_history_type_0_item import (
            RunResponseWithIncludesRunMessageHistoryType0Item,
        )
        from ..models.run_response_with_includes_sensitive_input_aliases_type_0 import (
            RunResponseWithIncludesSensitiveInputAliasesType0,
        )
        from ..models.run_response_with_includes_usage_metadata_type_0 import RunResponseWithIncludesUsageMetadataType0

        d = dict(src_dict)
        id = UUID(d.pop("id"))

        workflow_id = UUID(d.pop("workflow_id"))

        status = RunStatus(d.pop("status"))

        created_at = isoparse(d.pop("created_at"))

        def _parse_machine_id(data: object) -> None | Unset | UUID:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                machine_id_type_0 = UUID(data)

                return machine_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UUID, data)

        machine_id = _parse_machine_id(d.pop("machine_id", UNSET))

        def _parse_user_id(data: object) -> None | Unset | UUID:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                user_id_type_0 = UUID(data)

                return user_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UUID, data)

        user_id = _parse_user_id(d.pop("user_id", UNSET))

        def _parse_organization_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        organization_id = _parse_organization_id(d.pop("organization_id", UNSET))

        def _parse_error(data: object) -> list[str] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                error_type_0 = cast(list[str], data)

                return error_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None | Unset, data)

        error = _parse_error(d.pop("error", UNSET))

        def _parse_output_data(data: object) -> None | RunResponseWithIncludesOutputDataType0 | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                output_data_type_0 = RunResponseWithIncludesOutputDataType0.from_dict(data)

                return output_data_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | RunResponseWithIncludesOutputDataType0 | Unset, data)

        output_data = _parse_output_data(d.pop("output_data", UNSET))

        def _parse_input_attachment_ids(data: object) -> list[str] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                input_attachment_ids_type_0 = cast(list[str], data)

                return input_attachment_ids_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None | Unset, data)

        input_attachment_ids = _parse_input_attachment_ids(d.pop("input_attachment_ids", UNSET))

        def _parse_output_attachment_ids(data: object) -> list[str] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                output_attachment_ids_type_0 = cast(list[str], data)

                return output_attachment_ids_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None | Unset, data)

        output_attachment_ids = _parse_output_attachment_ids(d.pop("output_attachment_ids", UNSET))

        def _parse_run_message_history(
            data: object,
        ) -> list[RunResponseWithIncludesRunMessageHistoryType0Item] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                run_message_history_type_0 = []
                _run_message_history_type_0 = data
                for run_message_history_type_0_item_data in _run_message_history_type_0:
                    run_message_history_type_0_item = RunResponseWithIncludesRunMessageHistoryType0Item.from_dict(
                        run_message_history_type_0_item_data
                    )

                    run_message_history_type_0.append(run_message_history_type_0_item)

                return run_message_history_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[RunResponseWithIncludesRunMessageHistoryType0Item] | None | Unset, data)

        run_message_history = _parse_run_message_history(d.pop("run_message_history", UNSET))

        def _parse_input_values(data: object) -> None | RunResponseWithIncludesInputValuesType0 | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                input_values_type_0 = RunResponseWithIncludesInputValuesType0.from_dict(data)

                return input_values_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | RunResponseWithIncludesInputValuesType0 | Unset, data)

        input_values = _parse_input_values(d.pop("input_values", UNSET))

        def _parse_pool_ids(data: object) -> list[UUID] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                pool_ids_type_0 = []
                _pool_ids_type_0 = data
                for pool_ids_type_0_item_data in _pool_ids_type_0:
                    pool_ids_type_0_item = UUID(pool_ids_type_0_item_data)

                    pool_ids_type_0.append(pool_ids_type_0_item)

                return pool_ids_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[UUID] | None | Unset, data)

        pool_ids = _parse_pool_ids(d.pop("pool_ids", UNSET))

        def _parse_sensitive_input_aliases(
            data: object,
        ) -> None | RunResponseWithIncludesSensitiveInputAliasesType0 | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                sensitive_input_aliases_type_0 = RunResponseWithIncludesSensitiveInputAliasesType0.from_dict(data)

                return sensitive_input_aliases_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | RunResponseWithIncludesSensitiveInputAliasesType0 | Unset, data)

        sensitive_input_aliases = _parse_sensitive_input_aliases(d.pop("sensitive_input_aliases", UNSET))

        def _parse_usage_metadata(data: object) -> None | RunResponseWithIncludesUsageMetadataType0 | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                usage_metadata_type_0 = RunResponseWithIncludesUsageMetadataType0.from_dict(data)

                return usage_metadata_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | RunResponseWithIncludesUsageMetadataType0 | Unset, data)

        usage_metadata = _parse_usage_metadata(d.pop("usage_metadata", UNSET))

        def _parse_post_run_checks(data: object) -> list[RunPostRunCheckSnapshot] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                post_run_checks_type_0 = []
                _post_run_checks_type_0 = data
                for post_run_checks_type_0_item_data in _post_run_checks_type_0:
                    post_run_checks_type_0_item = RunPostRunCheckSnapshot.from_dict(post_run_checks_type_0_item_data)

                    post_run_checks_type_0.append(post_run_checks_type_0_item)

                return post_run_checks_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[RunPostRunCheckSnapshot] | None | Unset, data)

        post_run_checks = _parse_post_run_checks(d.pop("post_run_checks", UNSET))

        def _parse_session_id(data: object) -> None | Unset | UUID:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                session_id_type_0 = UUID(data)

                return session_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UUID, data)

        session_id = _parse_session_id(d.pop("session_id", UNSET))

        def _parse_session_alias(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        session_alias = _parse_session_alias(d.pop("session_alias", UNSET))

        def _parse_release_session_after(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        release_session_after = _parse_release_session_after(d.pop("release_session_after", UNSET))

        def _parse_started_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                started_at_type_0 = isoparse(data)

                return started_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        started_at = _parse_started_at(d.pop("started_at", UNSET))

        def _parse_ended_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                ended_at_type_0 = isoparse(data)

                return ended_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        ended_at = _parse_ended_at(d.pop("ended_at", UNSET))

        def _parse_included(data: object) -> list[IncludedResource] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                included_type_0 = []
                _included_type_0 = data
                for included_type_0_item_data in _included_type_0:
                    included_type_0_item = IncludedResource.from_dict(included_type_0_item_data)

                    included_type_0.append(included_type_0_item)

                return included_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[IncludedResource] | None | Unset, data)

        included = _parse_included(d.pop("included", UNSET))

        run_response_with_includes = cls(
            id=id,
            workflow_id=workflow_id,
            status=status,
            created_at=created_at,
            machine_id=machine_id,
            user_id=user_id,
            organization_id=organization_id,
            error=error,
            output_data=output_data,
            input_attachment_ids=input_attachment_ids,
            output_attachment_ids=output_attachment_ids,
            run_message_history=run_message_history,
            input_values=input_values,
            pool_ids=pool_ids,
            sensitive_input_aliases=sensitive_input_aliases,
            usage_metadata=usage_metadata,
            post_run_checks=post_run_checks,
            session_id=session_id,
            session_alias=session_alias,
            release_session_after=release_session_after,
            started_at=started_at,
            ended_at=ended_at,
            included=included,
        )

        run_response_with_includes.additional_properties = d
        return run_response_with_includes

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class PostRunCheckFileTargetMode(str, Enum):
    EXACT = "exact"
    LOOP_ITEMS = "loop_items"
    REGEX = "regex"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowTagCreate")


@_attrs_define
class WorkflowTagCreate:
    """Schema for creating a workflow tag

    Attributes:
        name (str):
        description (None | str | Unset):
        emoji (None | str | Unset):
        color (None | str | Unset):
        group_id (None | Unset | UUID): Optional group for mutual exclusivity
    """

    name: str
    description: None | str | Unset = UNSET
    emoji: None | str | Unset = UNSET
    color: None | str | Unset = UNSET
    group_id: None | Unset | UUID = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        emoji: None | str | Unset
        if isinstance(self.emoji, Unset):
            emoji = UNSET
        else:
            emoji = self.emoji

        color: None | str | Unset
        if isinstance(self.color, Unset):
            color = UNSET
        else:
            color = self.color

        group_id: None | str | Unset
        if isinstance(self.group_id, Unset):
            group_id = UNSET
        elif isinstance(self.group_id, UUID):
            group_id = str(self.group_id)
        else:
            group_id = self.group_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if emoji is not UNSET:
            field_dict["emoji"] = emoji
        if color is not UNSET:
            field_dict["color"] = color
        if group_id is not UNSET:
            field_dict["group_id"] = group_id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_emoji(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        emoji = _parse_emoji(d.pop("emoji", UNSET))

        def _parse_color(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        color = _parse_color(d.pop("color", UNSET))

        def _parse_group_id(data: object) -> None | Unset | UUID:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                group_id_type_0 = UUID(data)

                return group_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UUID, data)

        group_id = _parse_group_id(d.pop("group_id", UNSET))

        workflow_tag_create = cls(
            name=name,
            description=description,
            emoji=emoji,
            color=color,
            group_id=group_id,
        )

        workflow_tag_create.additional_properties = d
        return workflow_tag_create

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.paginated_response_with_includes_workflow_response import PaginatedResponseWithIncludesWorkflowResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    search: None | str | Unset = UNSET,
    created_at_from: datetime.datetime | None | Unset = UNSET,
    created_at_to: datetime.datetime | None | Unset = UNSET,
    updated_at_from: datetime.datetime | None | Unset = UNSET,
    updated_at_to: datetime.datetime | None | Unset = UNSET,
    tag_ids: None | str | Unset = UNSET,
    include_tags: bool | Unset = False,
    include: None | str | Unset = UNSET,
    skip: int | Unset = 0,
    limit: int | Unset = 100,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    json_search: None | str | Unset
    if isinstance(search, Unset):
        json_search = UNSET
    else:
        json_search = search
    params["search"] = json_search

    json_created_at_from: None | str | Unset
    if isinstance(created_at_from, Unset):
        json_created_at_from = UNSET
    elif isinstance(created_at_from, datetime.datetime):
        json_created_at_from = created_at_from.isoformat()
    else:
        json_created_at_from = created_at_from
    params["created_at_from"] = json_created_at_from

    json_created_at_to: None | str | Unset
    if isinstance(created_at_to, Unset):
        json_created_at_to = UNSET
    elif isinstance(created_at_to, datetime.datetime):
        json_created_at_to = created_at_to.isoformat()
    else:
        json_created_at_to = created_at_to
    params["created_at_to"] = json_created_at_to

    json_updated_at_from: None | str | Unset
    if isinstance(updated_at_from, Unset):
        json_updated_at_from = UNSET
    elif isinstance(updated_at_from, datetime.datetime):
        json_updated_at_from = updated_at_from.isoformat()
    else:
        json_updated_at_from = updated_at_from
    params["updated_at_from"] = json_updated_at_from

    json_updated_at_to: None | str | Unset
    if isinstance(updated_at_to, Unset):
        json_updated_at_to = UNSET
    elif isinstance(updated_at_to, datetime.datetime):
        json_updated_at_to = updated_at_to.isoformat()
    else:
        json_updated_at_to = updated_at_to
    params["updated_at_to"] = json_updated_at_to

    json_tag_ids: None | str | Unset
    if isinstance(tag_ids, Unset):
        json_tag_ids = UNSET
    else:
        json_tag_ids = tag_ids
    params["tag_ids"] = json_tag_ids

    params["include_tags"] = include_tags

    json_include: None | str | Unset
    if isinstance(include, Unset):
        json_include = UNSET
    else:
        json_include = include
    params["include"] = json_include

    params["skip"] = skip

    params["limit"] = limit

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/workflows",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> HTTPValidationError | PaginatedResponseWithIncludesWorkflowResponse | None:
    if response.status_code == 200:
        response_200 = PaginatedResponseWithIncludesWorkflowResponse.from_dict(response.json())

        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[HTTPValidationError | PaginatedResponseWithIncludesWorkflowResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    search: None | str | Unset = UNSET,
    created_at_from: datetime.datetime | None | Unset = UNSET,
    created_at_to: datetime.datetime | None | Unset = UNSET,
    updated_at_from: datetime.datetime | None | Unset = UNSET,
    updated_at_to: datetime.datetime | None | Unset = UNSET,
    tag_ids: None | str | Unset = UNSET,
    include_tags: bool | Unset = False,
    include: None | str | Unset = UNSET,
    skip: int | Unset = 0,
    limit: int | Unset = 100,
) -> Response[HTTPValidationError | PaginatedResponseWithIncludesWorkflowResponse]:
    """List Workflows

     List all workflows for the authenticated organization.

    Supports pagination and returns workflows ordered by updated_at descending.

    Use `tag_ids` to filter workflows that have ALL specified tags (AND logic).
    Use `include=tags,post_run_checks` to include related workflow resources.
    Legacy `include_tags=true` remains supported, but `include` wins when both are provided.

    Args:
        search (None | str | Unset): Search workflows by id or name (case-insensitive substring
            match)
        created_at_from (datetime.datetime | None | Unset): Filter workflows created at or after
            this ISO timestamp (UTC)
        created_at_to (datetime.datetime | None | Unset): Filter workflows created at or before
            this ISO timestamp (UTC)
        updated_at_from (datetime.datetime | None | Unset): Filter workflows updated at or after
            this ISO timestamp (UTC)
        updated_at_to (datetime.datetime | None | Unset): Filter workflows updated at or before
            this ISO timestamp (UTC)
        tag_ids (None | str | Unset): Comma-separated list of tag UUIDs. Returns workflows with
            ALL specified tags (AND logic).
        include_tags (bool | Unset): Include tags in the response Default: False.
        include (None | str | Unset): Comma-separated list of related resources to include.
            Allowed values: tags, post_run_checks. If provided, overrides legacy include_tags.
        skip (int | Unset):  Default: 0.
        limit (int | Unset):  Default: 100.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | PaginatedResponseWithIncludesWorkflowResponse]
    """

    kwargs = _get_kwargs(
        search=search,
        created_at_from=created_at_from,
        created_at_to=created_at_to,
        updated_at_from=updated_at_from,
        updated_at_to=updated_at_to,
        tag_ids=tag_ids,
        include_tags=include_tags,
        include=include,
        skip=skip,
        limit=limit,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    search: None | str | Unset = UNSET,
    created_at_from: datetime.datetime | None | Unset = UNSET,
    created_at_to: datetime.datetime | None | Unset = UNSET,
    updated_at_from: datetime.datetime | None | Unset = UNSET,
    updated_at_to: datetime.datetime | None | Unset = UNSET,
    tag_ids: None | str | Unset = UNSET,
    include_tags: bool | Unset = False,
    include: None | str | Unset = UNSET,
    skip: int | Unset = 0,
    limit: int | Unset = 100,
) -> HTTPValidationError | PaginatedResponseWithIncludesWorkflowResponse | None:
    """List Workflows

     List all workflows for the authenticated organization.

    Supports pagination and returns workflows ordered by updated_at descending.

    Use `tag_ids` to filter workflows that have ALL specified tags (AND logic).
    Use `include=tags,post_run_checks` to include related workflow resources.
    Legacy `include_tags=true` remains supported, but `include` wins when both are provided.

    Args:
        search (None | str | Unset): Search workflows by id or name (case-insensitive substring
            match)
        created_at_from (datetime.datetime | None | Unset): Filter workflows created at or after
            this ISO timestamp (UTC)
        created_at_to (datetime.datetime | None | Unset): Filter workflows created at or before
            this ISO timestamp (UTC)
        updated_at_from (datetime.datetime | None | Unset): Filter workflows updated at or after
            this ISO timestamp (UTC)
        updated_at_to (datetime.datetime | None | Unset): Filter workflows updated at or before
            this ISO timestamp (UTC)
        tag_ids (None | str | Unset): Comma-separated list of tag UUIDs. Returns workflows with
            ALL specified tags (AND logic).
        include_tags (bool | Unset): Include tags in the response Default: False.
        include (None | str | Unset): Comma-separated list of related resources to include.
            Allowed values: tags, post_run_checks. If provided, overrides legacy include_tags.
        skip (int | Unset):  Default: 0.
        limit (int | Unset):  Default: 100.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | PaginatedResponseWithIncludesWorkflowResponse
    """

    return sync_detailed(
        client=client,
        search=search,
        created_at_from=created_at_from,
        created_at_to=created_at_to,
        updated_at_from=updated_at_from,
        updated_at_to=updated_at_to,
        tag_ids=tag_ids,
        include_tags=include_tags,
        include=include,
        skip=skip,
        limit=limit,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    search: None | str | Unset = UNSET,
    created_at_from: datetime.datetime | None | Unset = UNSET,
    created_at_to: datetime.datetime | None | Unset = UNSET,
    updated_at_from: datetime.datetime | None | Unset = UNSET,
    updated_at_to: datetime.datetime | None | Unset = UNSET,
    tag_ids: None | str | Unset = UNSET,
    include_tags: bool | Unset = False,
    include: None | str | Unset = UNSET,
    skip: int | Unset = 0,
    limit: int | Unset = 100,
) -> Response[HTTPValidationError | PaginatedResponseWithIncludesWorkflowResponse]:
    """List Workflows

     List all workflows for the authenticated organization.

    Supports pagination and returns workflows ordered by updated_at descending.

    Use `tag_ids` to filter workflows that have ALL specified tags (AND logic).
    Use `include=tags,post_run_checks` to include related workflow resources.
    Legacy `include_tags=true` remains supported, but `include` wins when both are provided.

    Args:
        search (None | str | Unset): Search workflows by id or name (case-insensitive substring
            match)
        created_at_from (datetime.datetime | None | Unset): Filter workflows created at or after
            this ISO timestamp (UTC)
        created_at_to (datetime.datetime | None | Unset): Filter workflows created at or before
            this ISO timestamp (UTC)
        updated_at_from (datetime.datetime | None | Unset): Filter workflows updated at or after
            this ISO timestamp (UTC)
        updated_at_to (datetime.datetime | None | Unset): Filter workflows updated at or before
            this ISO timestamp (UTC)
        tag_ids (None | str | Unset): Comma-separated list of tag UUIDs. Returns workflows with
            ALL specified tags (AND logic).
        include_tags (bool | Unset): Include tags in the response Default: False.
        include (None | str | Unset): Comma-separated list of related resources to include.
            Allowed values: tags, post_run_checks. If provided, overrides legacy include_tags.
        skip (int | Unset):  Default: 0.
        limit (int | Unset):  Default: 100.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | PaginatedResponseWithIncludesWorkflowResponse]
    """

    kwargs = _get_kwargs(
        search=search,
        created_at_from=created_at_from,
        created_at_to=created_at_to,
        updated_at_from=updated_at_from,
        updated_at_to=updated_at_to,
        tag_ids=tag_ids,
        include_tags=include_tags,
        include=include,
        skip=skip,
        limit=limit,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    search: None | str | Unset = UNSET,
    created_at_from: datetime.datetime | None | Unset = UNSET,
    created_at_to: datetime.datetime | None | Unset = UNSET,
    updated_at_from: datetime.datetime | None | Unset = UNSET,
    updated_at_to: datetime.datetime | None | Unset = UNSET,
    tag_ids: None | str | Unset = UNSET,
    include_tags: bool | Unset = False,
    include: None | str | Unset = UNSET,
    skip: int | Unset = 0,
    limit: int | Unset = 100,
) -> HTTPValidationError | PaginatedResponseWithIncludesWorkflowResponse | None:
    """List Workflows

     List all workflows for the authenticated organization.

    Supports pagination and returns workflows ordered by updated_at descending.

    Use `tag_ids` to filter workflows that have ALL specified tags (AND logic).
    Use `include=tags,post_run_checks` to include related workflow resources.
    Legacy `include_tags=true` remains supported, but `include` wins when both are provided.

    Args:
        search (None | str | Unset): Search workflows by id or name (case-insensitive substring
            match)
        created_at_from (datetime.datetime | None | Unset): Filter workflows created at or after
            this ISO timestamp (UTC)
        created_at_to (datetime.datetime | None | Unset): Filter workflows created at or before
            this ISO timestamp (UTC)
        updated_at_from (datetime.datetime | None | Unset): Filter workflows updated at or after
            this ISO timestamp (UTC)
        updated_at_to (datetime.datetime | None | Unset): Filter workflows updated at or before
            this ISO timestamp (UTC)
        tag_ids (None | str | Unset): Comma-separated list of tag UUIDs. Returns workflows with
            ALL specified tags (AND logic).
        include_tags (bool | Unset): Include tags in the response Default: False.
        include (None | str | Unset): Comma-separated list of related resources to include.
            Allowed values: tags, post_run_checks. If provided, overrides legacy include_tags.
        skip (int | Unset):  Default: 0.
        limit (int | Unset):  Default: 100.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | PaginatedResponseWithIncludesWorkflowResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            search=search,
            created_at_from=created_at_from,
            created_at_to=created_at_to,
            updated_at_from=updated_at_from,
            updated_at_to=updated_at_to,
            tag_ids=tag_ids,
            include_tags=include_tags,
            include=include,
            skip=skip,
            limit=limit,
        )
    ).parsed
